package com.huimv.admin.service.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.admin.common.token.TokenSign;
import com.huimv.admin.common.utils.*;
import com.huimv.admin.entity.BaseLocation;
import com.huimv.admin.entity.BaseProcess;
import com.huimv.admin.entity.BillPcr;
import com.huimv.admin.entity.BillPersonnelAdmission;
import com.huimv.admin.mapper.BaseLocationMapper;
import com.huimv.admin.mapper.BaseProcessMapper;
import com.huimv.admin.mapper.BillPcrMapper;
import com.huimv.admin.mapper.BillPersonnelAdmissionMapper;
import com.huimv.admin.service.IBillPersonnelAdmissionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Map;

/**
 * <p>
 * 入场申请 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-07-21
 */
@Service
public class BillPersonnelAdmissionServiceImpl extends ServiceImpl<BillPersonnelAdmissionMapper, BillPersonnelAdmission> implements IBillPersonnelAdmissionService {
    @Resource
    private BillPersonnelAdmissionMapper billPersonnelAdmissionMapper;
    @Resource
    private BaseProcessMapper baseProcessMapper;
    @Resource
    private BillPcrMapper billPcrMapper;
    @Resource
    private BaseLocationMapper baseLocationMapper;

    @Autowired
    private UploadImage uploadImage;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result addAdmission(HttpServletRequest httpServletRequest, String admissionType, String vistitType, String userName, String phone, String workName, String papersType, String papersCode, MultipartFile imgUrl, String vistitDate, String vistitReson, String destId, String destName) throws IOException {
        int farmId = Integer.parseInt(TokenSign.getFarmIds(httpServletRequest));
        Integer userId = TokenSign.getMemberIdByJwtToken(httpServletRequest);
        if (!PhoneNumberValidator.isValidPhoneNumber(phone)) {
            return new Result(10001,"手机号格式错误！",false);
        }
        if (papersType.equals("0")){
            if (!IDCardValidator.isValidIDCard(papersCode)){
                return new Result(10001,"身份证号格式错误!",false);
            }
        }
        BillPersonnelAdmission billPersonnelAdmission = new BillPersonnelAdmission();
        billPersonnelAdmission.setAdmissionType(Integer.parseInt(admissionType));
        billPersonnelAdmission.setVistitType(Integer.parseInt(vistitType));
        billPersonnelAdmission.setDestId(Integer.parseInt(destId));
        billPersonnelAdmission.setDestName(destName);
        billPersonnelAdmission.setAdmissionUserId(userId);
        billPersonnelAdmission.setAdmissionUserName(userName);
        billPersonnelAdmission.setVistitDate(LocalDateTime.parse(vistitDate));
        billPersonnelAdmission.setSubDate(LocalDateTime.now());
        billPersonnelAdmission.setPhone(phone);
        billPersonnelAdmission.setWorkName(workName);
        billPersonnelAdmission.setPapersType(Integer.parseInt(papersType));
        billPersonnelAdmission.setPapersCode(papersCode);
        billPersonnelAdmission.setImgUrl(uploadImage.getImageCom(imgUrl));
        billPersonnelAdmission.setVistitReson(vistitReson);
        billPersonnelAdmission.setFarmId(farmId);
        BaseProcess baseProcess = new BaseProcess();
        baseProcess.setDestName(destName);
        baseProcess.setDestId(Integer.parseInt(destId));
        baseProcess.setAdmissionUserId(userId);
        baseProcess.setAdmissionUserName(userName);
        baseProcess.setSubDate(DateTime.now());
        baseProcess.setCurrentStatus("0");
        //获取位置
        BaseLocation baseLocation = baseLocationMapper.selectOne(new QueryWrapper<BaseLocation>().eq("farm_id", farmId).eq("vistit_type", vistitType).eq("parent_id", 0));
        baseProcess.setCurrentLocation(baseLocation.getLocationName());
        baseProcess.setCurrentLocationId(baseLocation.getId().toString());
        baseProcess.setFarmId(farmId);
        baseProcessMapper.insert(baseProcess);
        billPersonnelAdmission.setProcessId(baseProcess.getId());
        billPersonnelAdmissionMapper.insert(billPersonnelAdmission);
        return Result.SUCCESS();
    }

    @Override
    public Result listPersonalAdmission(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageNo = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        Page<BillPersonnelAdmission> page = new Page<>(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        Page<BillPersonnelAdmission> billPersonnelAdmissionPage = billPersonnelAdmissionMapper.selectPage(page, new QueryWrapper<BillPersonnelAdmission>().eq("admission_user_id", TokenSign.getMemberIdByJwtToken(httpServletRequest)).orderByDesc("sub_date"));
        return new Result(ResultCode.SUCCESS,billPersonnelAdmissionPage);
    }

    @Override
    public Result listAdmission(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageNo = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String type = paramsMap.get("type");
        QueryWrapper<BillPersonnelAdmission> queryWrapper = new QueryWrapper<>();
        Integer userType = TokenSign.getUserTypeByJwtToken(httpServletRequest);
        if (userType == 3){
            //生物安全负责人
            queryWrapper.eq("admission_type",0);
        }else if (userType == 2){
            //紧急入场
            queryWrapper.ne("admission_type",0);
        }
        switch (type){
            case "1":
                queryWrapper.eq("bill_status",0);
                break;
            case "2":
                queryWrapper.eq("bill_status",1);
                break;
            case "3":
                queryWrapper.eq("bill_status",2);
                break;
            default: break;
        }
        queryWrapper.orderByDesc("sub_date");
        Page<BillPersonnelAdmission> page = new Page<>(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        Page<BillPersonnelAdmission> billPersonnelAdmissionPage = billPersonnelAdmissionMapper.selectPage(page,queryWrapper);
        return new Result(ResultCode.SUCCESS,billPersonnelAdmissionPage);
    }

    @Override
    @Transactional
    public Result handleAdmission(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String id = paramsMap.get("id");
        BillPersonnelAdmission billPersonnelAdmission = billPersonnelAdmissionMapper.selectById(id);
        Integer userType = TokenSign.getUserTypeByJwtToken(httpServletRequest);
        billPersonnelAdmission.setPassDate(LocalDateTime.now());
        billPersonnelAdmission.setPassUserId(TokenSign.getMemberIdByJwtToken(httpServletRequest));
        billPersonnelAdmission.setPassUserName(TokenSign.getUserNameByJwtToken(httpServletRequest));
        BaseProcess baseProcess = baseProcessMapper.selectById(billPersonnelAdmission.getProcessId());
        if (type.equals("0")){
            //拒绝
            billPersonnelAdmission.setBillStatus(2);
            baseProcess.setProcessType(2);
        }else {
            //通过
            billPersonnelAdmission.setBillStatus(1);
            //获取pcr位置
            String laseLocationId = baseProcess.getCurrentLocationId();
            BaseLocation laseLocation = baseLocationMapper.selectById(laseLocationId);
            BaseLocation baseLocation = baseLocationMapper.selectById(laseLocation.getNextId());
            if (StringUtils.isBlank(baseProcess.getUpdateDate())){
                baseProcess.setUpdateDate(LocalDateTime.now().toString());
            }else {
                baseProcess.setUpdateDate(baseProcess.getUpdateDate()+","+LocalDateTime.now().toString());
            }
            baseProcess.setCurrentStatus("1,0");
            baseProcess.setCurrentLocation(baseProcess.getCurrentLocation()+","+baseLocation.getLocationName());
            baseProcess.setCurrentLocationId(baseProcess.getCurrentLocationId() +","+baseLocation.getId());
            if (userType == 3){
                BillPcr billPcr = new BillPcr();
                billPcr.setVistitType(billPersonnelAdmission.getVistitType());
                billPcr.setDestName(billPersonnelAdmission.getDestName());
                billPcr.setDestId(billPersonnelAdmission.getDestId());
                billPcr.setAdmissionUserId(billPersonnelAdmission.getAdmissionUserId());
                billPcr.setAdmissionUserName(billPersonnelAdmission.getAdmissionUserName());
                billPcr.setVistitDate(billPersonnelAdmission.getVistitDate());
                billPcr.setFarmId(billPersonnelAdmission.getFarmId());
                billPcr.setProcessId(billPersonnelAdmission.getProcessId());
                billPcr.setAdmissionId(Integer.parseInt(id));
                billPcr.setTestLocation(baseLocation.getLocationName());
                billPcr.setTestLocationId(baseLocation.getId());
                billPcrMapper.insert(billPcr);
            }else {
                //紧急入场通过先不处理
            }
        }


        return Result.SUCCESS();
    }


}
